// This file is part of seq192
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <iostream>
#include <nlohmann/json.hpp>

#include "cachefile.h"

using json = nlohmann::json;

extern string last_used_dir;
extern bool global_nsm_gui;

CacheFile::CacheFile(const string& a_name) :
    m_name(a_name)
{
}

CacheFile::~CacheFile()
{
}

bool
CacheFile::parse()
{

    /* open binary file */
    ifstream file (m_name.c_str());

    if (!file.is_open()) return false;

    json j;

    try {
        file >> j;
    } catch (json::exception& ex) {
        return false;
    }

    if (j["last_used_dir"].is_string()) last_used_dir.assign(j["last_used_dir"]);
    if (j["nsm_optional_gui"].is_boolean()) global_nsm_gui = j["nsm_optional_gui"];

    file.close();

    return true;
}


bool
CacheFile::write()
{
    ofstream file(m_name.c_str());

    if (!file.is_open()) return false;

    json j;

    j["last_used_dir"] = last_used_dir;
    j["nsm_optional_gui"] = global_nsm_gui;

    file << j.dump(4) << std::endl;

    file.close();

    return true;
}
